@@ -280,10 +280,10 @@ biasCorrect.list <- function(frc, hindcast, obs, method, scaleType, preci, prThr
   
   # Now real bias correction is executed.
   
-  memberIndex <- match('member', attributes(frcData)$dimensions)
+  memberIndex <- grepAndMatch('member', attributes(frcData)$dimensions)
   
   # For dataset that has a member part 
-  if (!is.na(memberIndex)) {
+  if (length(memberIndex) != 0) {
     # check if frcData and hindcastData has the same dimension and length.
     checkDimLength(frcData, hindcastData, dim = 'member')
     
